function CTCSegmentation(aImData, aVer, varargin)
% Saves segmentation results in frames with manual segmentation.
%
% The function segments only the images which have a segmentation ground
% truth, and saves label images to a faked tracking result in format of the
% ISBI Cell Tracking Challenges. No empty images are saved for un-segmented
% frames and no text file with track information is created. This can be
% useful for testing of segmentation parameters in cases where it takes a
% long time to produce complete tracking results.
%
% Inputs:
% aImData - ImData the image sequence to segment.
% aVer - Name of the generated tracking version.
%
% Property/Value inputs:
% NumImages - The number of images to segment. Only images with
%             segmentation ground truth are segmented. If this parameter is
%             set to NaN, all images with a segmentation ground truth will
%             be segmented.
% MostCells - If this parameter is set to true, the NumImages images with
%             most ground truth cells will be segmented. Otherwise the
%             images will be sampled evenly from the set of images with
%             ground truth segmentations.
% aScoringFunction - This parameter specifies the scoring function that
%                    should be optimized. The available options are 'SEG',
%                    and '(SEG+TRA)/2'. The SEG and TRA measures are the
%                    performance measures that were used to evaluate
%                    segmentation and tracking performance in the ISBI 2015
%                    Cell Tracking Challenge publication [1]. All images in
%                    the sequences have to be segmented to compute TRA, but
%                    for the SEG measure it is enough to segment the images
%                    which have ground truth segmentations. For SEG it is
%                    also possible to use only a subset of the images with
%                    segmentation ground truths. The default is 'SEG'.
% varargin - All valid settings parameters, and values that are fields in
%            AllSettings. The specified values override the values saved in
%            the settings files.
%
% References:
% [1] Ulman, V.; Maka, M.; Magnusson, K. E. G.; Ronneberger, O.; Haubold,
%     C.; Harder, N.; Matula, P.; Matula, P.; Svoboda, D.; Radojevic, M.;
%     Smal, I.; Rohr, K.; Jaldn, J.; Blau, H. M.; Dzyubachyk, O.;
%     Lelieveldt, B.; Xiao, P.; Li, Y.; Cho, S.-Y.; Dufour, A. C.;
%     Olivo-Marin, J.-C.; Reyes-Aldasoro, C. C.; Solis-Lemus, J. A.;
%     Bensch, R.; Brox, T.; Stegmaier, J.; Mikut, R.; Wolf, S.; Hamprecht,
%     F. A.; Esteves, T.; Quelhas, P.; Demirel, .; Malmstrm, L.; Jug, F.;
%     Tomancak, P.; Meijering, E.; Muoz-Barrutia, A.; Kozubek, M. &
%     Ortiz-de-Solorzano, C., An objective comparison of cell-tracking
%     algorithms, Nature methods, 2017, 14, 11411152

% Parse property/value inputs.
[evaluationArgs, settingsArgs] =...
    SelectArgs(varargin, {'NumImages', 'MostCells', 'ScoringFunction', 'Suffix', 'Suffix2'});
[aNumImages, aMostCells, aScoringFunction, aSuffix, aSuffix2] = GetArgs(...
    {'NumImages', 'MostCells', 'ScoringFunction', 'Suffix', 'Suffix2'},...
    {nan, false, 'SEG', '_GT', []}, true, evaluationArgs);

imData = aImData;

% Overwrite the saved settings with settings specified by the caller.
for i = 1:length(settingsArgs)/2
    imData.Set(settingsArgs{2*i-1}, settingsArgs{2*i});
end

switch aScoringFunction
    case {'SEG' '0.9*SEG+0.1*DET'}
        % Find the frames in which there are segmentation ground truths.
        seqDir = imData.GetSeqDir();
        gtPath = fullfile(imData.GetGroundTruthPath(aSuffix, true), 'SEG');
        
        % Find the frames with ground truth segmentations.
        gtImages = GetNames(gtPath, 'tif');
        gtStrings = regexp(gtImages, '(?<=man_seg_?)\d+', 'match', 'once');
        gtFramesWithDuplicates = cellfun(@str2double, gtStrings) + 1;
        gtFrames = unique(gtFramesWithDuplicates);
        
        if ~isnan(aNumImages)
            if aMostCells
                % Pick the images with the most cells.
                gtImagePaths = strcat(gtPath, filesep, gtImages);
                numCells = zeros(size(gtFrames));
                % Load the images to see how many cells they have.
                for i = 1:length(gtImages)
                    im = imread(gtImagePaths{i});
                    index = gtFrames == gtFramesWithDuplicates(i);
                    numCells(index) = numCells(index) + max(im(:));
                end
                [~,indices] = sort(numCells, 'descend');
                gtFrames = gtFrames(indices(1:aNumImages));
                gtFrames = sort(gtFrames);
            else
                if length(gtFrames) > aNumImages
                    % Sample images evenly from the first to the last image.
                    if aNumImages == 1
                        indices = 1;
                    else
                        stepLength = (length(gtFrames)-1) / (aNumImages-1);
                        indices = round(1 + (0:aNumImages-1) * stepLength);
                    end
                    gtFrames = gtFrames(indices);
                end
            end
        end
    case '0.45*SEG1+0.45*SEG2+0.1*DET'
        % Find the frames in which there are segmentation ground truths.
        
        assert(~aMostCells,...
            ['The alternative MostCells is not available for the '...
            'scoring function 0.45*SEG1+0.45*SEG2+0.1*DET.'])
        
        seqDir = imData.GetSeqDir();
        
        gtPath1 = fullfile(imData.GetGroundTruthPath(aSuffix, true), 'SEG');
        gtFrames1 = FindGtFrames(gtPath1);
        gtPath2 = fullfile(imData.GetGroundTruthPath(aSuffix2, true), 'SEG');
        gtFrames2 = FindGtFrames(gtPath2);
        
        assert(isempty(setdiff(gtFrames2, 1:length(gtFrames2))),...
            ['The frames of the second ground truth must be '...
            'consecutive and start with 1.'])
        
        numFramesWithGt = length(union(gtFrames1, gtFrames2));
        gtFrames = FindFramesToSegment(gtFrames1,...
            length(gtFrames2),...
            min(length(gtFrames1) + aNumImages, numFramesWithGt));
    case '(SEG+TRA)/2'
        gtFrames = 1:imData.sequenceLength;
    otherwise
        error('Unknown scoring function %s', aScoringFunction)
end

resPath = fullfile(....
    imData.GetCellDataDir('Version', aVer),...
    'RES',...
    [seqDir, '_RES']);

if exist(resPath, 'dir')
    % Remove old files.
    fclose('all'); % rmdir can fail because files are open in Matlab.
    rmdir(resPath, 's')
end
mkdir(resPath)

if imData.sequenceLength <= 1000
    fmt = 'mask%03d.tif';
else
    digits = ceil(log10(imData.sequenceLength));
    fmt = ['mask%0' num2str(digits) 'd.tif'];
end

% Pre-compute the min and max of the image sequence so that they are stored
% permanently in imData. If the computation is performed inside the
% for-loop, the computation is performed in each iteration and the values
% are not stored outside the for-loop.
imData.ComputeSequenceMinAndMax()

parfor i = 1:length(gtFrames)
    fprintf('Segmenting frame %d / %d\n', i, length(gtFrames))
    t = gtFrames(i);
    
    % Perform segmentation.
    if imData.GetDim() == 2
        blobs = Segment_generic(imData, t);
    else  % 3D
        blobs = Segment_generic3D(imData, t);
    end
    
    % Write label images to a tracking result folder.
    imPath = fullfile(resPath, sprintf(fmt, t-1));
    imageSize = [imData.imageHeight, imData.imageWidth, imData.numZ];
    im = ReconstructSegmentsBlob(blobs, imageSize);
    if size(im,3) == 1  % 2D data.
        imwrite(uint16(im), imPath, 'Compression', 'lzw')
    else  % 3D data.
        for slice = 1:size(im,3)
            imwrite(uint16(squeeze(im(:,:,slice))), imPath,...
                'WriteMode', 'append',...
                'Compression', 'lzw')
        end
    end
end

% Convert to tracking format if the scoring function is '0.9*SEG+0.1*DET'.
if strcmp(aScoringFunction, '0.9*SEG+0.1*DET') ||...
        strcmp(aScoringFunction, '0.45*SEG1+0.45*SEG2+0.1*DET')
    totalCellCount = 0;
    frameCellCounts = zeros(1, imData.sequenceLength);
    for i = 1:length(gtFrames)
        fprintf('Converting frames to tracking format %d / %d\n', i, length(gtFrames))
        t = gtFrames(i);

        imPath = fullfile(resPath, sprintf(fmt, t-1));
        
        % Read label image.
        if imData.numZ == 1  % 2D data.
            im = imread(imPath);
        else  % 3D data.
            im = zeros(imData.imageHeight, imData.imageWidth, imData.numZ, 'uint16');
            if exist(imPath, 'file')
                for z = 1:imData.numZ
                    im(:,:,z) = imread(imPath, z);
                end
            end
        end
        
        % Convert labels to unique cell identifiers.
        gtz = im > 0;
        if ~any(gtz)
            continue
        end
        frameCellCounts(t) = max(im(gtz));
        im(gtz) = im(gtz) + totalCellCount;
        totalCellCount = totalCellCount + frameCellCounts(t);
        
        % Save the modified label image.
        if imData.numZ == 1  % 2D data.
            imwrite(im, imPath, 'Compression', 'lzw')
        else  % 3D data.
            for slice = 1:size(im,3)
                if slice == 1
                    % Overwrite the existing file.
                    imwrite(im(:,:,slice), imPath,...
                    'Compression', 'lzw')
                else
                    % Append to the new file.
                    imwrite(im(:,:,slice), imPath,...
                    'WriteMode', 'append',...
                    'Compression', 'lzw')
                end
            end
        end
    end
    
    % Create a res_track.txt file.
    fid = fopen(fullfile(resPath, 'res_track.txt'), 'w');
    cellIndex = 1;
    for t = 1:imData.sequenceLength
        for i = 1:frameCellCounts(t)
            fprintf(fid, '%d %d %d %d\r\n', cellIndex, t-1, t-1, 0);
            cellIndex = cellIndex + 1;
        end
    end
    fclose(fid);
end

fprintf('Done segmenting frames\n')
end